package org.websocketsexample;

import com.soasta.scripting.v1.*;

import org.websocketsexample.WebSocketsExampleCommHandler;


public class WebSocketsExample
{ 
  public WebSocketsExample()
  {    
  }

  // This method is called from the SOASTA Script.
  // It passes in the SOASTA API and the address of the WebSockets
  // server that is to be used.
  public void execute(ScriptEnvironment env, String server) throws Throwable
  {
    // Get the current Result object from the API.
    Result result = env.getContext().getResult();
      
    // Catch all exceptions.
    try
    {
      // Get the current Composition object from the API.
      Composition composition = env.getContext().getComposition();
      
      // Create our WebSockets communication handler.
      // The address of the WebSockets server to use is passed in as a parameter
      // from the calling SOASTA Script.
      WebSocketsExampleCommHandler commHandler = new WebSocketsExampleCommHandler(result, server);
      
      // We'll put all of our "foreground work" into a Transaction in the Result.
      SimulatedTransactionInfo simulatedTransactionInfo = result.createSimulatedTransactionInfo();
      simulatedTransactionInfo.parentPathType = "Clip";
      simulatedTransactionInfo.parentPath = ".";
      simulatedTransactionInfo.name = "Foreground Transaction";
      simulatedTransactionInfo.repeatIndex = -1;
      simulatedTransactionInfo.deltaFromScheduledStart = 0;
      
      // Start the Transaction.
      result.recordTransactionStart(simulatedTransactionInfo);
      
      // Connect to the server.
      result.postMessage(ResultLevel.INFO, "Foreground: Connecting to: " + server);
      commHandler.connectBlocking();
      result.postMessage(ResultLevel.INFO, "Foreground: Have connected.");
      
      // Send message #1 to the server.
      result.postMessage(ResultLevel.INFO, "Foreground: Sending Message 1...");
      commHandler.send("Message 1");
      result.postMessage(ResultLevel.INFO, "Foreground: Have sent Message 1.");
      
      // Send message #2 to the server.
      result.postMessage(ResultLevel.INFO, "Foreground: Sending Message 2...");
      commHandler.send("Message 2");
      result.postMessage(ResultLevel.INFO, "Foreground: Have sent Message 2.");
      
      // Send message #3 to the server.
      result.postMessage(ResultLevel.INFO, "Foreground: Sending Message 3...");
      commHandler.send("Message 3");
      result.postMessage(ResultLevel.INFO, "Foreground: Have sent Message 3.");
		
      // End the Transaction.
      simulatedTransactionInfo.completionType = "Success";
      result.recordTransactionCompletion(simulatedTransactionInfo);
      
      // We'll now just wait forever for the user to stop the Composition.
      // In the meantime, the background will be receiving messages.
      // Obviously, this is not a real-world implementation, but an example.
      // In the real world you would probably wait on a certain received
      // message or something, I'm guessing.
      result.postMessage(ResultLevel.INFO, "Foreground: Waiting for Composition to be ended...");
      
      for (;;)
      {
        if (composition.isEnding())
          break;
  
        Thread.sleep(1000);
      }

      // The Composition is ending, so we'll end too.
      result.postMessage(ResultLevel.INFO, "Foreground: Done waiting for Composition to be ended.");

      // Close the connection.
      result.postMessage(ResultLevel.INFO, "Foreground: Closing...");
      commHandler.closeBlocking();
      result.postMessage(ResultLevel.INFO, "Foreground: Have closed.");
    }
    catch (Throwable throwable)
    {
      // We got an unexpected exception.
      result.postMessage(ResultLevel.ERROR, "Caught unexpected exception in Java.", throwable.toString());
    }
  }
}
