package org.websocketsexample;

import java.net.URI;
import java.net.URISyntaxException;

import com.soasta.scripting.v1.*;

import org.java_websocket.client.WebSocketClient;
import org.java_websocket.drafts.Draft;
import org.java_websocket.drafts.Draft_10;
import org.java_websocket.framing.Framedata;
import org.java_websocket.handshake.ServerHandshake;



// This class does the actual WebSockets communication.
// It uses the library at java-websocket.org.
public class WebSocketsExampleCommHandler extends WebSocketClient
{
  // Constructor.
  public WebSocketsExampleCommHandler(Result result, String server) throws Exception
  {
    super(new URI( server ), new Draft_10());
    
    m_result = result;
  }
  
  @Override
  public void onOpen( ServerHandshake handshakedata )
  {
    // We have connected with the server.
    try
    {
      // In our case we just log this event to the current Result.
      m_result.postMessage(ResultLevel.INFO, "CommHandler: Connection has been opened to: " + getURI().toString());
    }
    catch (Exception e)
    {
      m_result.postMessage(ResultLevel.ERROR, "CommHandler: Exception in onOpen: " + e.toString());
    }
  }

  @Override
  public void onMessage( String message )
  {
    // We have received a push message from the server.
    try
    {
      // Log this event to the current Result.
      m_result.postMessage(ResultLevel.INFO, "CommHandler: Have received a message.", message);
      
      // Create a simulated Target for this message.
      SimulatedTargetInfo simulatedTargetInfo = m_result.createSimulatedTargetInfo();
      simulatedTargetInfo.name = "My WebSockets Target";
      simulatedTargetInfo.repositoryPath = "/somewhere";
      simulatedTargetInfo.repositoryName = "MyRepoTarget";
      
      // Set up the basic information that CloudTest is to record for this Message.
      SimulatedActionInfo simulatedActionInfo = m_result.createSimulatedActionInfo();
      simulatedActionInfo.name = "Message " + m_messageNumber++;
      simulatedActionInfo.url = getURI().toString();
      simulatedActionInfo.textSent = null;
      simulatedActionInfo.textReceived = message;      
      simulatedActionInfo.repeatIndex = -1;
      simulatedActionInfo.operation = "WebSockets";
      simulatedActionInfo.failureText = null;
      simulatedActionInfo.validationPassed = true;
      simulatedActionInfo.completionType = "Success";

      // Set up the statistics that CloudTest is to record in the Result for this Message.
      // In this example, these are dummy values.  Presumably you would compute as many
      // of these as you can and that are appropriate for your situation.
      simulatedActionInfo.duration = 1234;
      simulatedActionInfo.deltaFromScheduledStart = 0;
      simulatedActionInfo.bytesSent = 0;
      simulatedActionInfo.bytesReceived = 123;
      simulatedActionInfo.totalPortRetryCount = 0;
      simulatedActionInfo.totalPortRetryTime = 0;
      simulatedActionInfo.connectionEstablishTime = 0;
      simulatedActionInfo.sendTime = 0;
      simulatedActionInfo.receiveTime = 456;
      simulatedActionInfo.totalTime = 789;
      simulatedActionInfo.ttfb = 12;
      simulatedActionInfo.ttlb = 34;
      simulatedActionInfo.dnsLookupDuration = 0;
      simulatedActionInfo.waitForConnectionPoolTime = 0;
      
      // Check for the special case of Message 2.
      if (message.contains("Message 2"))
      {
        // Message 2 is put into a different Group than the Script.
        // Just an example to show recording the message elsewhere
        // other than where the calling CloudTest Script is located.
        simulatedActionInfo.parentPathType = "Clip";
        simulatedActionInfo.parentPath = "Group1";
      }
      else
      {
        // This is not the special case of Message 2, so we just
        // record the message in the same container as the calling Script.
        simulatedActionInfo.parentPathType = null;
        simulatedActionInfo.parentPath = null;
      }

      // Record the message into the current Result.
      m_result.recordActionCompletion(simulatedTargetInfo, simulatedActionInfo);
    }
    catch (Throwable e)
    {
      m_result.postMessage(ResultLevel.ERROR, "CommHandler: Exception in onMessage: " + e.toString());
    }
  }

  @Override
  public void onClose( int code, String reason, boolean remote )
  {
    // The connection to the server is being closed.
    try
    {
      // In our case we just log this event to the current Result.
      m_result.postMessage(ResultLevel.INFO, "CommHandler: Connection closed by " + ( remote ? "remote peer" : "us" ) + ", reason: \"" + (reason == null ? "null" : reason) + "\"" + ", code=" + code);
    }
    catch (Exception e)
    {
      m_result.postMessage(ResultLevel.ERROR, "CommHandler: Exception in onClose: " + e.toString());
    }
  }

  @Override
  public void onError( Exception ex )
  {
    // There was some sort of communication error with the server.
    try
    {
      // In our case we just log this event to the current Result.
      m_result.postMessage(ResultLevel.ERROR, "CommHandler: Received error: " + ex.toString());
    }
    catch (Exception e)
    {
      m_result.postMessage(ResultLevel.ERROR, "CommHandler: Exception in onError: " + e.toString());
    }
  }
  
  
  private int m_messageNumber = 1;
  private Result m_result;
}
